@@ -1,6 +1,6 @@
 #!/bin/env python3
 
-# Copyright Louis Paternault 2017
+# Copyright Louis Paternault 2017-2022
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as
@@ -17,7 +17,7 @@
 
 """Command line client to :mod:`evariste`"""
 
-from evariste.evs.compile.__main__ import main
+from .evs.compile.__main__ import main
 
 if __name__ == "__main__":
     main()
