@@ -1,4 +1,4 @@
-# Copyright Louis Paternault 2017-2021
+# Copyright Louis Paternault 2017-2022
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as
@@ -18,20 +18,20 @@
 Implements a caching mechanism, to be able to store data between successive
 runs of Évariste.
 """
-
+import abc
 import os
 import pickle
 
-from evariste.shared import Shared, SharedDeepDict
-from evariste.utils import expand_path
-from evariste.errors import EvaristeError
+from .errors import EvaristeError
+from .shared import Shared, SharedDeepDict
+from .utils import expand_path
 
 
 class CacheError(EvaristeError):
     """Exception related to cache."""
 
 
-class AbstractCache:
+class AbstractCache(metaclass=abc.ABCMeta):
     """Parent class of all cache classes."""
 
     #: Version of cache format. If the version read from cache is different
@@ -41,11 +41,13 @@ class AbstractCache:
     def __init__(self, *, setup, builder, **data):
         self.shared = Shared(setup=setup, builder=builder, **data)
 
+    @abc.abstractmethod
     def close(self):
         """Close cache."""
         raise NotImplementedError
 
     @classmethod
+    @abc.abstractmethod
     def clear(cls, cachedir):
         """Delete cache.
 
