@@ -1,4 +1,4 @@
-# Copyright Louis Paternault 2020-2021
+# Copyright Louis Paternault 2020-2022
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as
@@ -19,7 +19,7 @@ import operator
 
 import argdispatch
 
-from evariste import plugins
+from ... import plugins
 
 
 def do_list(args):
@@ -48,6 +48,7 @@ def do_list(args):
         formatter = (
             """{plugin_type} {plugin.keyword} {plugin.__module__} "{description}" """
         )
+        print("""# Type Keyword Module Description""")
     else:
         formatter = "{plugin.keyword}"
 
@@ -58,7 +59,10 @@ def do_list(args):
             plugin_type = plugin.plugin_type
         else:
             plugin_type = "none"
-        description = plugin.__doc__.split("\n")[0]
+        if isinstance(plugin.__doc__, str):
+            description = plugin.__doc__.split("\n")[0]
+        else:
+            description = ""
         print(
             formatter.format(
                 plugin=plugin, plugin_type=plugin_type, description=description
