@@ -1,4 +1,4 @@
-# Copyright Louis Paternault 2015-2021
+# Copyright Louis Paternault 2015-2022
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as
@@ -14,7 +14,40 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 
-"""Plugin loader"""
+"""Plugin base class and plugin loader
+
+Every plugin is a subclass of :class:`Plugin` (see :ref:`write` for more information).
+
+The :class:`Loader` class finds and loads the plugins.
+
+Constants
+---------
+
+.. autodata:: MANDATORY_PLUGINS
+
+:class:`Plugin`
+---------------
+
+.. autoclass:: Plugin
+   :members:
+
+:class:`Loader`
+---------------
+
+.. autoclass:: Loader
+   :members: get_plugin, iter, values, items, applyiterhook, match
+
+Functions
+---------
+
+.. autofunction:: find_plugins
+
+Exceptions
+----------
+
+.. autoclass:: NoMatch
+
+"""
 
 import collections
 import contextlib
@@ -23,19 +56,22 @@ import logging
 import os
 import shlex
 import sys
+import typing
 
-from evariste import errors, utils, setup
-from evariste import hooks
-import evariste
+from .. import errors, hooks, setup, utils
+from ..shared import Shared
 
-LOGGER = logging.getLogger(evariste.__name__)
-DEFAULT_PLUGINS = {
+LOGGER = logging.getLogger("evariste")
+
+#: Set of mandatory plugins: plugins that are loaded by default, and cannot be disabled.
+MANDATORY_PLUGINS = {
     "action.cached",
     "action.directory",
     "action.noplugin",
     "action.raw",
     "changed",
     "logging",
+    "tree",
 }
 
 
@@ -57,11 +93,11 @@ class SameKeyword(errors.EvaristeError):
     def __init__(self, keyword, plugin1, plugin2):
         super().__init__()
         self.keyword = keyword
-        self.plugins = (plugin1, plugin2)
+        self._plugins = (plugin1, plugin2)
 
     def __str__(self):
         # pylint: disable=line-too-long
-        return f"""Plugins '{self.plugins[0].__name__}' and '{self.plugins[1].__name__}' have the same keyword '{self.keyword}'."""
+        return f"""Plugins '{self._plugins[0].__name__}' and '{self._plugins[1].__name__}' have the same keyword '{self.keyword}'."""
 
 
 class NotAPlugin(errors.EvaristeError):
@@ -76,7 +112,7 @@ class NotAPlugin(errors.EvaristeError):
             """Class '{obj.__module__}.{obj.__name__}' is not a plugin """
             """(it should inherit from """
             """'{superclass.__module__}.{superclass.__name__}')."""
-        ).format(obj=self.obj, superclass=PluginBase)
+        ).format(obj=self.obj, superclass=Plugin)
 
 
 class PluginNotFound(errors.EvaristeError):
@@ -91,38 +127,67 @@ class PluginNotFound(errors.EvaristeError):
 
 
 @functools.total_ordering
-class PluginBase:
-    """Plugin base: all imported plugins must be subclasses of this class."""
+class Plugin:
+    """Plugin base: all imported plugins must be subclasses of this class.
+
+    See :ref:`write` to see how to write a new plugin.
+
+    :param Shared shared: The object :ref:`shared <shared>` among plugins.
+    """
 
     # pylint: disable=too-few-public-methods
 
-    keyword = None
-    priority = 0
-    default_setup = {}
-    global_default_setup = {}
-    plugin_type = None
-    hooks = []
-    depends = {}
+    #: Keyword plugin, used to reference it: it is used
+    #: to :ref:`enable plugins in the setup file <plugins>`,
+    #: to name its section in the :ref:`setup file <setup>`, etc.
+    keyword: typing.Union[None, str] = None
+
+    #: When Évariste has to choose *one* plugin among several one,
+    #: it chooses the one with higher priority.
+    priority: int = 0
+
+    #: Default value for section ``self.keyword`` in the :ref:`setup file <setup>`.
+    #: It may be overwritten by data provided by user in the :ref:`setup file <setup>`.
+    #: See :ref:`default_setup`.
+    default_setup: typing.Dict[str, str] = {}
+
+    #: Default values for setup file. See :ref:`default_setup`.
+    global_default_setup: typing.Dict[str, typing.Dict[str, str]] = {}
+
+    #: Type of the plugin.
+    #: Plugins of the same type gather some common behaviour.
+    plugin_type: str = ""
+
+    #: Set of plugins this plugin depends on (as a list of :class:`str`):
+    #: when this plugin is enabled, those plugins are enabled as well.
+    depends: typing.Set[str] = {}
 
     def __init__(self, shared):
-        self.shared = shared
+        #: Common data shared with every :class:`Tree` and :class:`~evariste.plugins.Plugin`
+        #: of this :class:`~evariste.builder.Builder`.
+        self.shared: Shared = shared
+
+        #: Same as :attr:`Plugin.shared`, but from this plugin point of view:
+        #: see :meth:`evariste.shared.Shared.get_plugin_view` and :ref:`plugin_local`.
         self.local = shared.get_plugin_view(self.keyword)
+
         self._set_default_setup()
 
     def _set_default_setup(self):
         """Set default value for this plugin setup, if necessary."""
         default = setup.Setup()
         for parent in reversed(self.__class__.mro()):  # pylint: disable=no-member
-            if hasattr(parent, "default_setup"):
+            if hasattr(parent, "global_default_setup"):
                 default.update(parent.global_default_setup)
+            if hasattr(parent, "default_setup"):
                 default.update({self.keyword: parent.default_setup})
         self.shared.setup.fill_blanks(default)
 
-    def match(self, value, *args, **kwargs):  # pylint: disable=unused-argument
+    def match(self, value, *args, **kwargs) -> bool:  # pylint: disable=unused-argument
         """Return ``True`` iff ``value`` matches ``self``.
 
-        Default is keyword match. This method can be overloaded by
-        subclasses.
+        Default is keyword match.
+        This method can be overloaded by subclasses.
         """
         return value == self.keyword
 
@@ -138,10 +203,13 @@ class PluginBase:
         return priority < other_priority
 
 
-def find_plugins(libdirs=None):
+def find_plugins(
+    libdirs: typing.Union[typing.Iterable[str], None] = None
+) -> typing.Iterator[Plugin]:
     """Iterate over available plugins.
 
-    :param list libdirs: Additional list of directories where plugins can be found.
+    :param typing.Iterable[str] libdirs:
+        Additional iterable of directories where plugins can be found.
     """
     if libdirs is None:
         libdirs = []
@@ -150,20 +218,20 @@ def find_plugins(libdirs=None):
     path.extend(
         [
             os.path.join(utils.expand_path(item), "plugins")
-            for item in [".evariste", "~/.config/evariste", "~/.evariste"]
+            for item in [".evariste", "~/.local/evariste", "~/.evariste"]
         ]
     )
     path.extend([utils.expand_path(item) for item in libdirs])
     path.extend([os.path.join(item, "evariste") for item in sys.path])
 
     yielded = set()
-    for module in utils.iter_modules(path, "evariste.", LOGGER.debug):
+    for module in utils.iter_modules(path, "evariste."):
         for attr in dir(module):
             if attr.startswith("_"):
                 continue
             obj = getattr(module, attr)
 
-            if isinstance(obj, type) and issubclass(obj, PluginBase):
+            if isinstance(obj, type) and issubclass(obj, Plugin):
                 if obj.keyword is None:
                     continue
                 if obj in yielded:
@@ -194,14 +262,6 @@ def find_plugins_sorted(libdirs=None):
     return plugindict
 
 
-def get_plugins(self, *_args, **_kwargs):
-    """Given a :class:`PluginLoader` object, return itself.
-
-    Useful to give hooks the plugins object.
-    """
-    return self
-
-
 def get_libdirs(libdirs):
     """Convert `libdirs` setup option (as a string) to a list of path (as strings)."""
     if not libdirs:
@@ -213,8 +273,23 @@ def get_libdirs(libdirs):
     return []
 
 
-class PluginLoader:
-    """Load plugins"""
+class Loader:
+    """Load plugins
+
+    :param evariste.shared.Shared shared: The :ref:`shared <shared>` object among plugins.
+
+    The constructor (:meth:`Loader.__init__`):
+
+    - reads the :ref:`setup file <setup>` (looking for the :ref:`libdirs <libdirs>` option);
+    - search all plugins (subclasses of :class:`Plugin`);
+    - instanciate:
+
+      - the :const:`mandatory plugins <MANDATORY_PLUGINS>`,
+      - those enabled in the :ref:`setup file <setup>`,
+      - and their dependencies;
+
+    - store them in some attribute, so that they can be accessed later.
+    """
 
     # pylint: disable=too-few-public-methods
 
@@ -230,28 +305,28 @@ class PluginLoader:
         )
 
         # Step 0: Convert setup options in a list of keyword
-        if self.shared.setup["setup"]["enable_plugins"] is None:
-            self.shared.setup["setup"]["enable_plugins"] = []
+        if self.shared.setup["setup"]["plugins"] is None:
+            self.shared.setup["setup"]["plugins"] = []
         else:
-            if isinstance(self.shared.setup["setup"]["enable_plugins"], str):
-                self.shared.setup["setup"]["enable_plugins"] = shlex.split(
-                    self.shared.setup["setup"]["enable_plugins"]
+            if isinstance(self.shared.setup["setup"]["plugins"], str):
+                self.shared.setup["setup"]["plugins"] = shlex.split(
+                    self.shared.setup["setup"]["plugins"]
                 )
-            elif isinstance(self.shared.setup["setup"]["enable_plugins"], list):
+            elif isinstance(self.shared.setup["setup"]["plugins"], list):
                 pass
             else:
                 raise ValueError(
                     (
                         # pylint: disable=consider-using-f-string, line-too-long
-                        "'Setup[setup][enable_plugins]' should be a string or a list (is now {}: '{}')."
+                        "'Setup[setup][plugins]' should be a string or a list (is now {}: '{}')."
                     ).format(
-                        type(self.shared.setup["setup"]["enable_plugins"]),
-                        self.shared.setup["setup"]["enable_plugins"],
+                        type(self.shared.setup["setup"]["plugins"]),
+                        self.shared.setup["setup"]["plugins"],
                     )
                 )
 
         # Step 1: Add enabled plugins
-        for keyword in self.shared.setup["setup"]["enable_plugins"]:
+        for keyword in self.shared.setup["setup"]["plugins"]:
             try:
                 self.available.get_subkey(keyword)
             except KeyError as error:
@@ -264,11 +339,16 @@ class PluginLoader:
                 if "enable" in self.shared.setup[plugin.keyword]:
                     if utils.yesno(self.shared.setup[plugin.keyword]["enable"]):
                         to_load.add(plugin.keyword)
+                    elif plugin.keyword in MANDATORY_PLUGINS:
+                        LOGGER.warning(
+                            f"""Attemp to disable mandatory plugin "{plugin.keyword}" ignored."""
+                        )
+                        continue
                     else:
                         to_load.discard(plugin.keyword)
 
         # Step 3: Add default plugins
-        to_load |= DEFAULT_PLUGINS
+        to_load |= MANDATORY_PLUGINS
 
         # Step 4: Manage dependencies
         to_process = to_load.copy()
@@ -294,92 +374,104 @@ class PluginLoader:
             processed.add(keyword)
 
         # Step 5: Instanciate plugins
-        self.plugins = collections.defaultdict(dict)
+        self._plugins = collections.defaultdict(dict)
+        self._hooks = collections.defaultdict(lambda: collections.defaultdict(set))
         for keyword in to_load:
-            plugin = self.available.get_subkey(keyword)
-            self.plugins[plugin.plugin_type][keyword] = plugin(self.shared)
+            plugin = self.available.get_subkey(keyword)(self.shared)
+            self._plugins[plugin.plugin_type][keyword] = plugin
+            for hooktype, name, function in hooks.iter_hooks(plugin):
+                self._hooks[hooktype][name].add(function)
 
-    def iter_plugins(self, plugin_type=None):
-        """Iterate over plugins."""
-        if plugin_type is None:
-            for ptype in self.plugins:
-                for keyword in self.get_plugin_type(ptype):
-                    yield self.get_plugin_type(ptype)[keyword]
-        else:
-            for keyword in self.get_plugin_type(plugin_type):
-                yield self.get_plugin_type(plugin_type)[keyword]
+    def values(
+        self, plugin_type: typing.Optional[str] = None
+    ) -> typing.Iterable[Plugin]:
+        """Iterate over plugins.
 
-    def _iter_hooks(self, cls, name, *args, **kwargs):
-        for plugin in self.iter_plugins():
-            for hook in plugin.hooks:
-                if not issubclass(hook, cls):
-                    continue
-                if hook.hookname != name:
-                    continue
-                yield hook(plugin, *args, **kwargs)
-
-    @contextlib.contextmanager
-    def apply_context_hook(self, hookname, *args, **kwargs):
-        """Apply a context hook (that is, run stuff before and after a `with` context).
-
-        :param str hookname: Name of the hook to apply.
-        :param list args: List of parameters to pass to hooks constructor
-        :param dict kwargs: Dictionary of parameters to pass to hooks and
-            ``function``.
+        :param typing.Optional[str] plugin_type: See :meth:`Loader.iter`.
         """
-        with contextlib.ExitStack() as stack:
-            for hook in self._iter_hooks(hooks.ContextHook, hookname, *args, **kwargs):
-                stack.enter_context(hook)
-            yield
+        if plugin_type is None:
+            for ptype in self._plugins:
+                yield from self._plugins[ptype].values()
+        else:
+            yield from self._plugins[plugin_type].values()
 
-    def apply_method_hook(self, hookname, function, *args, **kwargs):
-        """Apply a method hook (that is, run stuff before and after a method call).
+    def applymethodhook(
+        self, hookname: str, function: typing.Callable, *args, **kwargs
+    ):
+        """Apply a :ref:`method hook <methodhook>`.
+
+        That is, run stuff before and after a method call.
 
         :param str hookname: Name of the hook to apply.
         :param function function: Function to which the hook is to be applied.
-        :param list args: List of parameters to pass to hooks and
-            ``function``.
-        :param dict kwargs: Dictionary of parameters to pass to hooks and
-            ``function``.
+        :param list args: List of parameters to pass to hooks and ``function``.
+        :param dict kwargs: Dictionary of parameters to pass to hooks and ``function``.
         """
         with contextlib.ExitStack() as stack:
-            for hook in self._iter_hooks(hooks.ContextHook, hookname, *args, **kwargs):
-                stack.enter_context(hook)
-            for hook in self._iter_hooks(hooks.MethodHook, hookname):
-                function = hook.wrap(function)
+            for hook in sorted(self._hooks["contexthook"][hookname], key=str):
+                stack.enter_context(hook(*args, **kwargs))
+            for hook in sorted(self._hooks["methodhook"][hookname], key=str):
+                function = hook(function)
+
             return function(*args, **kwargs)
 
-    def get_plugin(self, keyword):
-        """Return the plugin with the given keyword."""
-        for plugin in self.iter_plugins():
+    def applyiterhook(self, hookname: str, *args, **kwargs) -> typing.Iterable:
+        """Apply an :ref:`iteration hook <iterhook>`.
+
+        Run every fonction hooked to this name (they should be iterators),
+        and iterate over the chain of those iterators.
+
+        :param str hookname: Name of the hook to apply.
+        :param list arg: Positional arguments passed to the hooks.
+        :param dict kwargs: Named arguments passed to the hooks.
+        """
+        for hook in sorted(self._hooks["iterhook"][hookname], key=str):
+            yield from hook(*args, **kwargs)
+
+    def get_plugin(self, keyword: str) -> Plugin:
+        """Return the plugin with the given keyword.
+
+        :raises NoMatch: If no (loaded) plugin was found with this keyword.
+        """
+        for plugin in self.values():
             if plugin.keyword == keyword:
                 return plugin
-        raise NoMatch(keyword, sorted(plugin.keyword for plugin in self.iter_plugins()))
+        raise NoMatch(keyword, sorted(plugin.keyword for plugin in self.values()))
 
-    def get_plugin_type(self, plugin_type):
-        """Return a dictionary of plugins of the given type."""
-        return self.plugins[plugin_type]
+    def items(
+        self, plugin_type: typing.Optional[str] = None
+    ) -> typing.Iterable[typing.Tuple[str, Plugin]]:
+        """Iterate over plugin keywords.
 
-    def iter_pluginkeywords(self, plugin_type=None):
-        """Iterate over plugin keywords."""
-        for plugin in self.iter_plugins(plugin_type):
+        :param typing.Optional[str] plugin_type: See :meth:`Loader.iter`.
+        """
+        for plugin in self.values(plugin_type):
             yield plugin.keyword
 
-    def match(self, plugin_type, value):
+    def match(self, plugin_type: typing.Optional[str], value) -> Plugin:
         """Return the first plugin matching ``value``.
 
-        A plugin ``Foo`` matches ``value`` if ``Foo.match(value)`` returns
-        True.
+        A plugin ``Foo`` matches ``value`` if ``Foo.match(value)`` returns ``True``.
+
+        :param typing.Optional[str] plugin_type: See :meth:`Loader.iter`.
         """
-        for plugin in sorted(self.get_plugin_type(plugin_type).values(), reverse=True):
+        for plugin in sorted(self.values(plugin_type), reverse=True):
             if plugin.match(value):
                 return plugin
-        raise NoMatch(value, sorted(self.iter_keywords(plugin_type)))
+        raise NoMatch(value, sorted(self.iter(plugin_type)))
 
-    def iter_keywords(self, plugin_type=None):
-        """Iterate over keywords"""
+    def iter(
+        self, plugin_type: typing.Union[str, None] = None
+    ) -> typing.Iterable[Plugin]:
+        """Iterate over keywords.
+
+        :param typing.Optional[str] plugin_type: Type of the plugins to iterate over.
+
+            - if ``None``, iterate over keywords of every (loaded) plugins;
+            - else, iterate over keywords of plugins of this given type only.
+        """
         if plugin_type is None:
-            for ptype in self.plugins:
-                yield from self.get_plugin_type(ptype)
+            for ptype in self._plugins:
+                yield from self._plugins[ptype]
         else:
-            yield from self.get_plugin_type(plugin_type)
+            yield from self._plugins[plugin_type]
