@@ -17,7 +17,7 @@
 
 import logging
 
-from evariste.plugins.action import Action, Report
+from . import Action, Report
 
 LOGGER = logging.getLogger(__name__)
 
