@@ -1,4 +1,4 @@
-# Copyright Louis Paternault 2015-2021
+# Copyright Louis Paternault 2015-2022
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as
@@ -18,66 +18,76 @@
 It is used not to re-compile files that have not changed.
 """
 
-from datetime import datetime
+import contextlib
+import functools
 import os
 import pathlib
+from datetime import datetime
 
-from evariste import hooks
-from evariste import plugins
-from evariste.tree import File
+from .. import plugins
+from ..hooks import contexthook, methodhook
+from ..tree import File
 
 
-class MakeArchive(hooks.MethodHook):
-    """Hook called before and after archive creation."""
+class Changed(plugins.Plugin):
+    """Evaluate and store modification times of files."""
 
-    # pylint: disable=too-few-public-methods
+    keyword = "changed"
+    default_setup = {"time": "vcs"}
 
-    hookname = "File.make_archive"
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self.files_to_compile = set()
+        self.files_to_ignore = set()
 
-    def wrap(self, function):
-        """Return a wrapped version of `function`, caching return value."""
+    def compile(self, tree):
+        """Return `True` iff file `tree` is to be compiled.
 
+        It is not compiled if:
+        - nothing has changed since last compilation;
+        - it is a dependency of a file described in previous item.
+        """
+        return (tree.from_fs.resolve() in self.files_to_compile) and (
+            tree.from_fs.resolve() not in self.files_to_ignore
+        )
+
+    # Hooks
+    @methodhook("File.make_archive")
+    def make_archive(self, function):
+        """Save archive path."""
+
+        @functools.wraps(function)
         def wrapped(tree, destdir):
             """Wrapped function."""
             # pylint: disable=unused-argument
             value = function(tree, destdir)
-            if self.plugin.compile(tree):
+            if self.compile(tree):
                 self.local.tree[tree.from_source]["archivepath"] = value
             return value
 
         return wrapped
 
-
-class BuildTreeItem(hooks.ContextHook):
-    """Hook called before and after the tree is built."""
-
-    # pylint: disable=too-few-public-methods
-
-    hookname = "Tree"
-
-    def __init__(self, plugin, tree, path, parent):
-        # pylint: disable=unused-argument
-        super().__init__(plugin)
-        self.tree = tree
-
-    def __exit__(self, *args, **kwargs):
+    @contexthook("Tree")
+    @contextlib.contextmanager
+    def build_tree_item(self, tree, *args, **kwargs):  # pylint: disable=unused-argument
         """Compare data with cache to see if a compilation is needed."""
-        if not isinstance(self.tree, File):
-            return
 
-        tree = self.tree
+        yield
+
+        if not isinstance(tree, File):
+            return
 
         # If the --always-compile argument was given to the command line call,
         # do compile.
         if self.shared.setup["arguments"]["always_compile"]:
-            self.plugin.files_to_compile.add(tree.from_fs.resolve())
+            self.files_to_compile.add(tree.from_fs.resolve())
             return
 
         # If tree is not in hash, compile it
         if (self.local.tree[tree.from_source] is None) or (
             self.local.tree[tree.from_source] == {}
         ):
-            self.plugin.files_to_compile.add(tree.from_fs.resolve())
+            self.files_to_compile.add(tree.from_fs.resolve())
             return
 
         # Otherwise, check if data hash has changed
@@ -91,32 +101,24 @@ class BuildTreeItem(hooks.ContextHook):
                     try:
                         file_mtime = datetime.fromtimestamp(os.path.getmtime(depend))
                     except FileNotFoundError:
-                        self.plugin.files_to_compile.add(tree.from_fs.resolve())
+                        self.files_to_compile.add(tree.from_fs.resolve())
                         return
                 if cache_mtime != file_mtime:
-                    self.plugin.files_to_compile.add(tree.from_fs.resolve())
+                    self.files_to_compile.add(tree.from_fs.resolve())
                     return
 
-
-class CompileFile(hooks.ContextHook):
-    """Hook called before and after compilation of one file."""
-
-    # pylint: disable=too-few-public-methods
-
-    hookname = "File.compile"
-
-    def __init__(self, plugin, tree):
-        super().__init__(plugin)
-        self.tree = tree
-
-    def __exit__(self, *args, **kwargs):
+    @contexthook("File.compile")
+    @contextlib.contextmanager
+    def file_compile(self, tree):
         """Store the modification time of file and its dependencies."""
-        if not isinstance(self.tree, File):
+
+        yield
+
+        if not isinstance(tree, File):
             return
 
-        tree = self.tree
         if not tree.report.success:
-            self.plugin.files_to_compile.discard(tree.from_fs.resolve())
+            self.files_to_compile.discard(tree.from_fs.resolve())
             return
 
         if self.local.tree[tree.from_source] is None:
@@ -140,31 +142,4 @@ class CompileFile(hooks.ContextHook):
             }
 
         # Mark dependencies as ignored (not to be compiled)
-        self.plugin.files_to_ignore.update(
-            [depend.resolve() for depend in tree.depends()]
-        )
-
-
-class Changed(plugins.PluginBase):
-    """Evaluate and store modification times of files."""
-
-    plugin_type = ""
-    keyword = "changed"
-    default_setup = {"time": "vcs"}
-    hooks = [BuildTreeItem, CompileFile, MakeArchive]
-
-    def __init__(self, *args, **kwargs):
-        super().__init__(*args, **kwargs)
-        self.files_to_compile = set()
-        self.files_to_ignore = set()
-
-    def compile(self, tree):
-        """Return `True` iff file `tree` is to be compiled.
-
-        It is not compiled if:
-        - nothing has changed since last compilation;
-        - it is a dependency of a file described in previous item.
-        """
-        return (tree.from_fs.resolve() in self.files_to_compile) and (
-            tree.from_fs.resolve() not in self.files_to_ignore
-        )
+        self.files_to_ignore.update([depend.resolve() for depend in tree.depends()])
