@@ -1,4 +1,4 @@
-# Copyright Louis Paternault 2015-2021
+# Copyright Louis Paternault 2015-2022
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as
@@ -13,17 +13,28 @@
 # You should have received a copy of the GNU Affero General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-"""Access to VCS (git, etc.) versionned files."""
+"""Access to VCS (git, etc.) versionned files.
 
-from datetime import datetime
+Every path processed here is a :class:`pathlib.Path` object.
+
+.. autoclass:: VCS
+   :members:
+   :special-members: __contains__
+
+"""
+
+import abc
 import os
 import pathlib
+import typing
+from datetime import datetime
 
-from evariste import errors, plugins
-from evariste.tree import Root
+from ...errors import EvaristeError
+from ...tree import Root
+from .. import Plugin
 
 
-class NoRepositoryError(errors.EvaristeError):
+class NoRepositoryError(EvaristeError):
     """No repository contains the given path."""
 
     def __init__(self, vcstype, directory):
@@ -36,35 +47,42 @@ class NoRepositoryError(errors.EvaristeError):
         return f"Could not find any {self.vcstype} repository containing directory '{self.directory}'."
 
 
-class VCS(plugins.PluginBase):
-    """Generic class to access to versionned files."""
+class VCS(Plugin, metaclass=abc.ABCMeta):
+    """Generic class to access to versionned files.
+
+    To write a new VCS plugin, one has to subclass this class, and implement every abstract method
+    (see for instance the implementation of :class:`evariste.plugin.vcs.git.Git`).
+    """
 
     plugin_type = "vcs"
     global_default_setup = {"setup": {"source": "."}}
 
     @property
-    def source(self):
+    def source(self) -> pathlib.Path:
         """Return an absolute version of source setup option."""
-        return pathlib.Path(self.shared.setup["setup"]["source"])
+        return pathlib.Path(self.shared.setup["setup"]["source"]).resolve()
 
-    def walk(self):
+    @abc.abstractmethod
+    def walk(self) -> typing.Iterable[pathlib.Path]:
         """Iterate versionned files, descendants of source (as defined by setup file)."""
         raise NotImplementedError()
 
-    def is_versionned(self, path):
+    @abc.abstractmethod
+    def __contains__(self, path: pathlib.Path) -> bool:
         """Return ``True`` iff ``path`` is versionned."""
         raise NotImplementedError()
 
     @property
-    def workdir(self):
+    @abc.abstractmethod
+    def workdir(self) -> pathlib.Path:
         """Return path of the root of the repository."""
         raise NotImplementedError()
 
-    def from_repo(self, path):
+    def from_repo(self, path: pathlib.Path) -> pathlib.Path:
         """Return ``path``, relative to the repository root."""
-        return os.path.relpath(path.resolve().as_posix(), self.workdir)
+        return path.resolve().relative_to(self.workdir)
 
-    def last_modified(self, path):
+    def last_modified(self, path: pathlib.Path) -> datetime:
         """Return the datetime of last modification."""
         # pylint: disable=no-self-use
-        return datetime.fromtimestamp(os.path.getmtime(str(path)))
+        return datetime.fromtimestamp(os.path.getmtime(path.as_posix()))
