@@ -1,4 +1,4 @@
-# Copyright Louis Paternault 2015-2021
+# Copyright Louis Paternault 2015-2022
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as
@@ -13,15 +13,23 @@
 # You should have received a copy of the GNU Affero General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-"""Access to git-versionned files."""
+"""Access to git-versionned files.
 
-from datetime import datetime
+.. autoclass:: Git
+   :members:
+   :special-members: __contains__
+"""
+
+import datetime
 import logging
 import os
+import pathlib
+import typing
+from datetime import datetime
 
 import git
 
-from evariste.plugins.vcs import VCS, NoRepositoryError
+from . import VCS, NoRepositoryError
 
 LOGGER = logging.getLogger(__name__)
 
@@ -36,7 +44,9 @@ class Git(VCS):
     def __init__(self, shared):
         super().__init__(shared)
         try:
-            self.repository = git.Repo(str(self.source), search_parent_directories=True)
+            self.repository = git.Repo(
+                self.source.as_posix(), search_parent_directories=True
+            )
         except git.InvalidGitRepositoryError as error:
             raise NoRepositoryError(self.keyword, self.source) from error
 
@@ -51,32 +61,39 @@ class Git(VCS):
             if not os.path.isfile(blob.path):
                 continue
             commit = next(self.repository.iter_commits(paths=blob.path, max_count=1))
-            last_modified[blob.path] = datetime.fromtimestamp(commit.committed_date)
+            last_modified[pathlib.Path(blob.path)] = datetime.fromtimestamp(
+                commit.committed_date
+            )
         # Files not committed yet
         for blob in self.repository.index.iter_blobs():
             if blob[1].path not in last_modified:
-                last_modified[blob[1].path] = super().last_modified(
-                    os.path.join(self.workdir, blob[1].path)
+                last_modified[pathlib.Path(blob[1].path)] = super().last_modified(
+                    self.workdir / blob[1].path
                 )
         LOGGER.info("Done")
         return last_modified
 
-    def walk(self):
-        source = self.source.resolve().as_posix()
+    def walk(self) -> typing.Iterable[pathlib.Path]:
         for entry in self._last_modified:
-            path = os.path.join(self.workdir, entry)
-            if path.startswith(source):
-                yield os.path.relpath(path, source)
+            try:
+                yield (self.workdir / entry).relative_to(self.source)
+            except ValueError:
+                # `self.source` is not a subpath of `path`
+                continue
 
-    def is_versionned(self, path):
-        return self.from_repo(path) in self._last_modified
+    def __contains__(self, path: pathlib.Path) -> bool:
+        try:
+            return self.from_repo(path) in self._last_modified
+        except ValueError:
+            # ``path`` is not a subpath of ``self.workdir``
+            return False
 
     @property
-    def workdir(self):
-        return self.repository.working_dir
+    def workdir(self) -> pathlib.Path:
+        return pathlib.Path(self.repository.working_dir)
 
-    def last_modified(self, path):
-        if not self.is_versionned(path):
+    def last_modified(self, path: pathlib.Path) -> datetime:
+        if path not in self:
             return super().last_modified(path)
 
         return self._last_modified[self.from_repo(path)]
