@@ -1,4 +1,4 @@
-# Copyright Louis Paternault 2015
+# Copyright Louis Paternault 2015-2022
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as
@@ -13,12 +13,16 @@
 # You should have received a copy of the GNU Affero General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-"""Dummy, do-nothing vcs. Used for tests."""
+"""Dummy, do-nothing vcs. Used for tests.
 
-from datetime import datetime
-import os
+.. autoclass:: NoneVCS
+   :members:
+   :special-members: __contains__
+"""
 
-from evariste.plugins.vcs import VCS
+import pathlib
+
+from . import VCS
 
 
 class NoneVCS(VCS):
@@ -29,15 +33,9 @@ class NoneVCS(VCS):
     def walk(self):
         yield from []
 
-    def is_versionned(self, path):
+    def __contains__(self, path: pathlib.Path) -> bool:
         return False
 
     @property
-    def workdir(self):
-        return os.getcwd()
-
-    def from_repo(self, path):
-        return os.path.relpath(path.resolve().as_posix(), self.workdir)
-
-    def last_modified(self, path):
-        return datetime.fromtimestamp(os.path.getmtime(str(path)))
+    def workdir(self) -> pathlib.Path:
+        return pathlib.Path.cwd()
