@@ -1,4 +1,4 @@
-# Copyright Louis Paternault 2015-2021
+# Copyright Louis Paternault 2015-2022
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as
@@ -13,21 +13,42 @@
 # You should have received a copy of the GNU Affero General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-"""A rag-bag of utility functions that did not fit anywhere else."""
+"""A rag-bag of utility functions that did not fit anywhere else.
+
+.. autoclass:: DeepDict
+   :members:
+
+.. autofunction:: smart_open
+
+.. autofunction:: yesno
+
+.. autofunction:: expand_path
+
+.. autoclass:: ChangeDir
+
+.. autofunction:: cached_iterator
+
+"""
+
+# Can be removed starting with python3.11
+from __future__ import annotations
 
-from collections import defaultdict
 import configparser
 import contextlib
 import functools
 import importlib
 import io
+import logging
 import os
 import pkgutil
 import re
 import shutil
 import sys
+import typing
+from collections import defaultdict
 
 RE_BRACKETS = re.compile("{[^{]*}")
+LOGGER = logging.getLogger(__name__)
 
 
 def copy(source, destination):
@@ -36,22 +57,6 @@ def copy(source, destination):
     shutil.copy2(source, destination)
 
 
-def copytree(source, destination):
-    """Recursively copy a tree.
-
-    Note: We do not use :func:`shutil.copytree` because it raises an error if
-    destination already exists, which we do not want.
-    """
-    os.makedirs(source, exist_ok=True)
-    for path in os.listdir(source):
-        sourceitem = os.path.join(source, path)
-        destitem = os.path.join(destination, path)
-        if os.path.isdir(sourceitem):
-            copytree(sourceitem, destitem)
-        else:
-            copy(sourceitem, destitem)
-
-
 def partial_format(string, *args, **kwargs):
     """Format string, leaving unfilled fields untouched.
 
@@ -68,13 +73,17 @@ def partial_format(string, *args, **kwargs):
 def smart_open(filename, mode="w", encoding="utf8"):
     """Open filename, standard output, or nothing.
 
-    If ``filename`` is:
 
-    * ``""`` (the empty string): return a fake file object, which is empty if
-      file is open for reading, can be written in if file is open for writing;
-    * ``"-"`` (a dash): read from standard input, or write to standard output
-      (depending on mode);
-    * any other: open the given file.
+    :param str filename: If ``filename`` is:
+
+        * ``""`` (the empty string): return a fake file object,
+            which is empty if file is open for reading,
+            and can be written in if file is open for writing (but content is then discarded);
+        * ``"-"`` (a dash):
+            read from standard input, or write to standard output (depending on mode);
+        * any other: open the given file.
+    :param str mode: Same as the ``mode`` parameter of :func:`open`.
+    :param str encoding: Same as the ``encoding`` parameter of :func:`open`.
     """
     if (not filename) or (not filename.strip()):
         with io.StringIO() as file:
@@ -89,8 +98,8 @@ def smart_open(filename, mode="w", encoding="utf8"):
             yield file
 
 
-def yesno(arg):
-    """Interpret some variable as a boolean.
+def yesno(arg: typing.Union[bool, str, int, None]) -> bool:
+    """Interpret some (mostly :class:`str`) variable as a boolean.
 
     >>> yesno("y")
     True
@@ -145,7 +154,10 @@ def expand_path(path):
 
 
 class DeepDict(defaultdict):
-    """Dictionary of dictionary of ... of dictionaries."""
+    """Dictionary of dictionary of ... of dictionaries.
+
+    All the dictionaries (expeted the last one) are :class:`collections.defaultdict` objects.
+    """
 
     def __init__(self, depth, dictionary=None):
         if dictionary is None:
@@ -162,7 +174,7 @@ class DeepDict(defaultdict):
             self[key].update(value)
 
     @classmethod
-    def from_configparser(cls, config):
+    def from_configparser(cls, config: configparser.ConfigParser) -> DeepDict:
         """Create a :class:`DeepDict` object from a :class:`configparser.ConfigParser` object."""
         dictionary = cls(2)
         for section in config.sections():
@@ -171,8 +183,8 @@ class DeepDict(defaultdict):
         return dictionary
 
     def get_subkey(self, subkey):
-        """Return the first `self[ANY][subkey]`, when `ANY` is any dictionary key."""
-        for key in self:
+        """Return the first ``self[ANY][subkey]``, when ``ANY`` is any dictionary key."""
+        for key in sorted(self):
             if subkey in self[key]:
                 return self[key][subkey]
         raise KeyError(subkey)
@@ -184,8 +196,11 @@ class DeepDict(defaultdict):
             return dict(self)
         return {key: vars(value) for (key, value) in self.items()}
 
-    def fill_blanks(self, other):
-        """Recursively copy values of `other` into `self`, if `self` is not defined."""
+    def fill_blanks(self, other: dict):
+        """Recursively copy values of ``other`` into ``self``.
+
+        The values are copied only if those of ``self`` are not defined.
+        """
         if self.depth == 1:
             for key in other:
                 if key not in self:
@@ -195,7 +210,7 @@ class DeepDict(defaultdict):
                 self[key].fill_blanks(other[key])
 
     def copy(self):
-        """Return a copy of `self`."""
+        """Return a copy of ``self``."""
         new = self.__class__(depth=self.depth)
         if self.depth == 1:
             for key in self:
@@ -206,14 +221,11 @@ class DeepDict(defaultdict):
         return new
 
 
-def iter_modules(path, prefix, logger=None):
+def iter_modules(path, prefix):
     """Iterate over modules.
 
     :arg list path: List of paths to look for modules in.
-    :arg str prefix: String to output on the front of every module name on
-        output.
-    :arg function logger: Function used to print logging error. May be `None`
-        to discard logging.
+    :arg str prefix: String to output on the front of every module name on output.
     """
     for module_finder, name, __is_pkg in pkgutil.walk_packages(path, prefix):
         if name in sys.modules:
@@ -228,8 +240,10 @@ def iter_modules(path, prefix, logger=None):
                 spec.loader.exec_module(module)
                 sys.modules[name] = module
             except ImportError as error:
-                if logger is not None:
-                    logger(f"[plugins] Could not load module {name}: {error}")
+                LOGGER.debug(f"[plugins] Could not load module {name}: {error}")
+                continue
+            except Exception as error:  #  pylint: disable=broad-except
+                LOGGER.debug(f"[plugins] Ignoring module {name}: {error}")
                 continue
         yield module
 
@@ -240,3 +254,18 @@ def read_config(filename, *args, **kwargs):
     with open(filename, encoding="utf8") as file:
         config.read_file(file, source=filename)
     return config
+
+
+def cached_iterator(func):
+    """Like :func:`functools.cache`, but for iterators.
+
+    That is, the first time the function is run, the returned iterable is stored (as a tuple),
+    and next calls to the function return this tuple.
+    """
+
+    @functools.wraps(func)
+    @functools.lru_cache()  # Can be replaced by @functools.cache once python3.8 is obsolete
+    def wrapped(*args, **kwargs):
+        return tuple(func(*args, **kwargs))
+
+    return wrapped
