@@ -1,4 +1,4 @@
-# Copyright Louis Paternault 2020-2021
+# Copyright Louis Paternault 2020-2022
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -24,8 +24,8 @@ import subprocess
 import sys
 import unittest
 
-from evariste.builder import Builder
 from evariste import utils
+from evariste.builder import Builder
 from evariste.setup import Setup
 
 SETUP = "evariste.setup"
@@ -94,10 +94,6 @@ class TestCompilation(unittest.TestCase, metaclass=DynamicTest):
                 # Run compilation
                 with Builder.from_setupname(SETUP) as builder:
                     builder.compile()  # pylint: disable=no-member
-                    for (
-                        renderer
-                    ) in builder.iter_renderers():  # pylint: disable=no-member
-                        renderer()
 
                 # Compare expected output to actual output
                 self.assertTreeEqual("dest", "expected")
