@@ -1,4 +1,4 @@
-# Copyright Louis Paternault 2015-2021
+# Copyright Louis Paternault 2015-2022
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as
@@ -34,7 +34,7 @@ class TestLibdirs(TestLoadedPlugins):
 
     def test_nolibdirs(self):
         """Test compilation without the libdirs option"""
-        self.assertSetEqual(self._loaded_plugins({}), self.default_plugins)
+        self.assertSetEqual(self._loaded_plugins({}), self.mandatory_plugins)
 
     def test_fail(self):
         """Test failure (plugin not found)."""
@@ -43,7 +43,7 @@ class TestLibdirs(TestLoadedPlugins):
                 {
                     "setup": {
                         "libdirs": absdir("plugins"),
-                        "enable_plugins": ["foo", "bar"],
+                        "plugins": ["foo", "bar"],
                     }
                 }
             )
@@ -53,9 +53,9 @@ class TestLibdirs(TestLoadedPlugins):
         with self.subTest("Without spaces"):
             self.assertSetEqual(
                 self._loaded_plugins(
-                    {"setup": {"libdirs": absdir("plugins"), "enable_plugins": ["foo"]}}
+                    {"setup": {"libdirs": absdir("plugins"), "plugins": ["foo"]}}
                 ),
-                set(["foo"]) | self.default_plugins,
+                set(["foo"]) | self.mandatory_plugins,
             )
 
         with self.subTest("With spaces"):
@@ -64,11 +64,11 @@ class TestLibdirs(TestLoadedPlugins):
                     {
                         "setup": {
                             "libdirs": f'"{absdir("plug ins")}"',
-                            "enable_plugins": ["bar"],
+                            "plugins": ["bar"],
                         }
                     }
                 ),
-                set(["bar"]) | self.default_plugins,
+                set(["bar"]) | self.mandatory_plugins,
             )
 
         with self.subTest("With and without spaces"):
@@ -79,9 +79,9 @@ class TestLibdirs(TestLoadedPlugins):
                             "libdirs": "{} {}".format(
                                 absdir(r"plug\ ins"), absdir("plugins")
                             ),
-                            "enable_plugins": ["foo", "bar"],
+                            "plugins": ["foo", "bar"],
                         }
                     }
                 ),
-                set(["bar", "foo"]) | self.default_plugins,
+                set(["bar", "foo"]) | self.mandatory_plugins,
             )
